module lapfilln
! fill empty 2-D bins by minimum output of Laplacian operator
  use laplacian
  use cgstep_mod
  use solver_smp_mod
  use solver_reg_mod
  use mask1
  use lint
  use rescale_bill

  implicit none
  contains
  subroutine lapfill               ( niter, n, yy, known)
    integer, intent (in) ::          niter, n(:)
    logical, dimension (:), pointer         ::     known
    ! mask for known
    real,    dimension (:), intent (in out) :: yy     ! model
    real,    dimension (:),allocatable                  :: zero   
    ! laplacian output
    allocate(zero(product(n)))
    call laplac_init (n)
    call mask1_init(known)
    zero = 0.      ! initialize
    call solver_smp ( m=yy,                &
                      d=zero,              &
                      Fop=laplac_lop,      &
                      stepper=cgstep,      &
                      niter=niter,         &
                      Kop=mask1_lop,       &
                      m0 = yy)
    call laplac_close ()
    ! garbage collection
    call cgstep_close ()
    ! garbage collection
    deallocate(zero)
  end subroutine lapfill

  subroutine laplfill (  data,model,maskl,n,o,d,niter,eps)
    integer, intent(in) ::                n(:), niter
    logical, dimension(:),pointer:: maskl
    real, dimension(:):: data,model,        o,d
    real            ::                                eps
    real, pointer :: coord(:,:),points(:)

    allocate(coord(count(.not. maskl),size(n)),points(count(.not. maskl)))
write(0,*)"laplfill 1"
call rmms(points,"point")
call rmms2(coord,"coord")
    call bin2points (data,n,o,d,.not. maskl,points,coord)
write(0,*)"laplfill 2"
    call lint_init(n,o,d,coord)
write(0,*)"laplfill 3"
    call laplac_init(n)
write(0,*)"laplfill 4"
    call solver_reg(m=model,                                    &
                    d=points,                                   &
                    Fop=lint_lop,                               &
                    Aop=laplac_lop,                             &
                    stepper=cgstep,                             &
                    nAop=(product(n)),                          &
                    niter=niter,                                &
                    eps=eps)
write(0,*)"laplfill 5"
    call lint_close()
write(0,*)"laplfill 6"
    call cgstep_close()
write(0,*)"laplfill 7"
    call laplac_close()
write(0,*)"laplfill 8"
    deallocate(points,coord)
  end subroutine laplfill
end module lapfilln
