module npef_find
! Find non-stationary prediction-error filter (helix magic)
  use rescale_bill
  use nhconest
  use cgstep_mod
  use solver_prc_mod
  use solver_reg_mod
  use ilaplacf
  use laplacf
  use dottest
  use error_check
  implicit none
contains
  
  subroutine npef_find   (pef,data,   mask,n,o,   d,a,   center,   gap,   npatch,   niter,eps,prec,epsl,nl)
    real,pointer    ::        data(:)
    logical, pointer::                mask(:), miss(:)
    integer         ::                     n(:),                                                        nl, i
    real            ::                       o(:),d(:),                                   eps,     epsl
    integer         :: bsize,                       a(:),center(:),gap(:),npatch(:),niter,    prec
    type(nfilter)   ::    pef
    real,   pointer :: pefcoeffs(:)
    integer, pointer:: patch(:)
    real, dimension(2):: dot1,dot2

!write(0,*)"nlnmpef_begin"
! __________________________________________________________
! ___________Precompute PEF info for different scales_______
! __________________________________________________________
!write(0,*)"nlnmpef 3"

!call lmms(maskbank,"mbank")
    allocate(miss(size(pef%mis)))
!    call pefcheck(pef)
!    call lmms(mask,"mask ")
    call bound_find(mask,n,a,center,gap,miss)
!    call boundbank_find(mask,n,a,center,gap,miss)
!    call pefcheck(pef)
    allocate(patch(size(pef%pch)))
    call createpatch(npatch,n,patch)
!    call imms(patch,"patch")
call lmms(miss,"miss ")
    pef%pch=patch
    pef%mis=miss
call lmms(pef%mis,"p%mis")
write(0,*) "pef%mis(2)",pef%mis(2)
!call imms(patchbank,"pbank")
!write(0,*)"nlnmpef 4"
    allocate(pefcoeffs(size(pef%hlx(1)%lag)*maxval(pef%pch)))
!call imms(patchbank,"pbank")
! __________________________________________________________
! ___________Pass precomputed stuff to operator init________
! __________________________________________________________

!call rmms(data,"data ")
!call pefcheck(pef)
!write(0,*) "size pef%mis", size(pef%mis),pef%mis(2)
!call pefcheck(pef)
!write(0,*)"size(pef%hlx(1)%lag)",size(pef%hlx(1)%lag)
       call nhconest_init(data,pef,size(pef%hlx(1)%lag))

! __________________________________________________________
! ___________Call solver for either preconditioned or_______
! ___________just regularized case._________________________
! __________________________________________________________
call rmms(data,"data ")
call rmms(pefcoeffs,"pefco")

       if (prec==1) then
          call ilaplacf_init(npatch,size(pef%hlx(1)%lag),epsl,a)
          !      call dot_test(nmhconest_new_lop,size(pefcoeffs),size(databank),dot1,dot2)
          !      call dot_test(ilaplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_prc(m=pefcoeffs,                                 &
                          d=-data,                                     &
                          Fop=nhconest_lop,                            &
                          Sop=ilaplacf_lop,                            &
                          stepper=cgstep,                              &
                          nSop=(maxval(pef%pch)*size(pef%hlx(1)%lag)), &
                          niter=niter,                                 &
                          eps=eps,                                     &
                          verb=.true.)
write(0,*)"before ilap_close"
          call ilaplacf_close()
       else
          call laplacf_init(npatch,size(pef%hlx(1)%lag))
                call dot_test(nhconest_lop,size(pefcoeffs),size(data),dot1,dot2)
                call dot_test(laplacf_lop,size(pefcoeffs),size(pefcoeffs),dot1,dot2)
          call solver_reg(m=pefcoeffs,                                 &
                          d=-data,                                     &
                          Fop=nhconest_lop,                            &
                          Aop=laplacf_lop,                             &
                          stepper=cgstep,                              &
                          nAop=(maxval(pef%pch)*size(pef%hlx(1)%lag)), &
                          niter=niter,                                 &
                          eps=eps,                                     &
                          verb=.true.)
          call laplacf_close()
       end if
write(0,*)"before nmh_close"
       call nhconest_close()
       call cgstep_close()
write(0,*)"after solver",i
call rmms(pefcoeffs,"pefca")
write(0,*)"PEF: pef"
call pefcheck(pef)       

    deallocate(pefcoeffs)
  end subroutine npef_find

end module npef_find
