module pef2fill

  use helix
  use peff2
  use cartesian
  use cgstep_mod
  use random
  use solver_smp_mod
  use solver_reg_mod
  use error_check
  use rescale_bill
  use mask1
  use dottest
  
  implicit none

contains

  subroutine dospeffill          ( data,model,pef1,pef2,iter,rmov,mmov)
    type(filter)                 ::           pef1,pef2
    real,dimension(:,:),pointer  :: data,model,               rmov,mmov
    real,dimension(:),allocatable :: data1,m
    integer                       ::                    iter, nd, n1,n2
    real, dimension(2)::dot1,dot2
    logical,dimension(:),pointer :: mask

    nd=size(data)
    n1=size(model,1)
    n2=size(model,2)
    allocate(data1(nd),m(nd),mask(nd))
    data1(1:nd)=reshape(data,(/nd/))
    mask=.true.
    where (data1/=0)
       mask=.false.
    end where
    m=reshape(model,(/ size(model) /))
    call spefcheck(pef1)
    call spefcheck(pef2)
    call rmms(data1,"data1")
    call rmms2(data, "data ")
    call rmms2(model,"model")
    call rmms(m,"m    ")
    call rmms2(mmov,"mmov ")
    call rmms2(rmov,"rmov ")
write(0,*)"mad_warped 1"
    call peff2_init(pef1,pef2)
write(0,*)"mad_warped 2",size(model),size(data)
    call dot_test(peff2_lop,size(model),size(data),dot1,dot2)
    call mask1_init(mask)
    call solver_smp(m=m,                                    &
         d=data1,                                   &
         Fop=peff2_lop,                               &
         stepper=cgstep,                             &
         niter=iter,                                &
         Kop=mask1_lop,   &
         m0=data1,                          &
         mmov=mmov,                    &
         rmov=rmov,                     &
         verb=.true.)
    call peff2_close()
    call cgstep_close()
    model=reshape(m,(/n1,n2/))

    deallocate(data1,m)

  end subroutine dospeffill

  subroutine dospeffilleps        ( data,model,pef1,pef2,iter,rmov,mmov,eps)
    type(filter)                 ::            pef1,pef2
    real,dimension(:,:),pointer  :: data,model,               rmov,mmov
    real,dimension(:),allocatable :: data1,m
    integer                       ::                    iter, nd, n1,n2
    real, dimension(2)::dot1,dot2
    logical,dimension(:),pointer :: mask
    real ::                                                             eps

    nd=size(data)
    n1=size(model,1)
    n2=size(model,2)
    allocate(data1(nd),m(nd),mask(nd))
    data1(1:nd)=reshape(data,(/nd/))
    mask=.false.
    where (data1/=0)
       mask=.true.
    end where
    m=reshape(model,(/ size(model) /))
    call spefcheck(pef1)
    call spefcheck(pef2)
    call rmms(data1,"data1")
    call rmms2(data, "data ")
    call rmms2(model,"model")
    call rmms(m,"m    ")
    call rmms2(mmov,"mmov ")
    call rmms2(rmov,"rmov ")
write(0,*)"mad_warped 1"
    call peff2_init(pef1,pef2)
write(0,*)"mad_warped 2",size(model),size(data)
    call dot_test(peff2_lop,size(model),size(data),dot1,dot2)
    call mask1_init(mask)
    call solver_reg(m=m,                                    &
         d=data1,                                   &
         Fop=mask1_lop, &
         Aop=peff2_lop,                               &
         naop=nd, &
         stepper=cgstep,                             &
         niter=iter,                                &
         m0=data1,                          &
         mmov=mmov,                    &
         rmov=rmov,                     &
         eps=eps, &
         verb=.true.)
    call peff2_close()
    call mask1_close()
    call cgstep_close()
    model=reshape(m,(/n1,n2/))

    deallocate(data1,m)

  end subroutine dospeffilleps

end module pef2fill
